@@ -4,6 +4,8 @@
 //! A library for defining enums that can be used in compact bit sets. It supports enums up to 128
 //! variants, and has a macro to use these sets in constants.
 //!
+//! For serde support, enable the `serde` feature.
+//!
 //! # Defining enums for use with EnumSet
 //!
 //! Enums to be used with [`EnumSet`] should be defined using `#[derive(EnumSetType)]`:
@@ -74,6 +76,7 @@
 #[cfg(test)] extern crate core;
 extern crate enumset_derive;
 extern crate num_traits;
+#[cfg(feature = "serde")] extern crate serde;
 
 pub use enumset_derive::*;
 mod enumset { pub use super::*; }
@@ -426,6 +429,24 @@ impl <T : EnumSetType + Debug> Debug for EnumSet<T> {
     }
 }
 
+#[cfg(feature = "serde")]
+impl <T : EnumSetType> serde::Serialize for EnumSet<T>
+    where T::Repr: serde::Serialize
+{
+    fn serialize<S: serde::Serializer>(&self, serializer: S) -> Result<S::Ok, S::Error> {
+        self.__enumset_underlying.serialize(serializer)
+    }
+}
+
+#[cfg(feature = "serde")]
+impl <'de, T : EnumSetType> serde::Deserialize<'de> for EnumSet<T>
+    where T::Repr: serde::Deserialize<'de>
+{
+    fn deserialize<D: serde::Deserializer<'de>>(deserializer: D) -> Result<Self, D::Error> {
+        T::Repr::deserialize(deserializer).map(|x| EnumSet { __enumset_underlying: x })
+    }
+}
+
 /// The iterator used by [`EnumSet`](./struct.EnumSet.html).
 #[derive(Copy, Clone, PartialOrd, Ord, PartialEq, Eq, Hash, Debug)]
 pub struct EnumSetIter<T : EnumSetType>(EnumSet<T>, u8);
@@ -526,6 +547,9 @@ macro_rules! enum_set {
 mod test {
     use super::*;
 
+    #[cfg(feature = "serde")]
+    extern crate bincode;
+
     mod enums {
         #[derive(::EnumSetType, Debug)]
         pub enum SmallEnum {
@@ -710,6 +734,15 @@ mod test {
                         _ => panic!("match fell through?"),
                     }
                 }
+
+                #[test]
+                #[cfg(feature = "serde")]
+                fn serialize_deserialize_test() {
+                    let value = $e::A | $e::C | $e::D | $e::F | $e::E | $e::G;
+                    let serialized = bincode::serialize(&value).unwrap();
+                    let deserialized = bincode::deserialize::<EnumSet<$e>>(&serialized).unwrap();
+                    assert_eq!(value, deserialized);
+                }
             }
         }
     }
